@@ -2,6 +2,7 @@ use crate::csv::Row;
 use crate::find;
 use crate::input::InputMode;
 use crate::view;
+use regex::Regex;
 use tui::buffer::Buffer;
 use tui::layout::Rect;
 use tui::style::{Color, Modifier, Style};
@@ -185,7 +186,9 @@ impl<'a> CsvTable<'a> {
                     .add_modifier(Modifier::BOLD);
             }
             match &state.finder_state {
-                FinderState::FinderActive(active) if (*hname).contains(active.target.as_str()) => {
+                // TODO: seems like doing a bit too much of heavy lifting of
+                // checking for matches (finder's work)
+                FinderState::FinderActive(active) if active.target.is_match(hname) => {
                     let mut highlight_style = style.fg(Color::Rgb(200, 0, 0));
                     if let Some(hl) = &active.found_record {
                         if let Some(row_index) = row_index {
@@ -197,11 +200,18 @@ impl<'a> CsvTable<'a> {
                             }
                         }
                     }
-                    let p_span = Span::styled(active.target.as_str(), highlight_style);
-                    let splitted = (*hname).split(active.target.as_str());
+                    // highlight parts that match
+                    let mut matches = active.target.find_iter(hname);
+                    let non_matches = active.target.split(hname);
                     let mut spans = vec![];
-                    for part in splitted {
+                    for part in non_matches {
                         let span = Span::styled(part, style);
+                        let cur_match = if let Some(m) = matches.next() {
+                            m.as_str()
+                        } else {
+                            ""
+                        };
+                        let p_span = Span::styled(cur_match, highlight_style);
                         spans.push(span);
                         spans.push(p_span.clone());
                     }
@@ -260,7 +270,10 @@ impl<'a> CsvTable<'a> {
         // Content of status line (separator already plotted elsewhere)
         let style = Style::default().fg(Color::Rgb(128, 128, 128));
         let mut content: String;
-        if let BufferState::Enabled(buffer_mode, buf) = &state.buffer_content {
+        if let Some(error) = &state.user_error {
+            content = error.to_owned();
+        }
+        else if let BufferState::Enabled(buffer_mode, buf) = &state.buffer_content {
             content = buf.to_owned();
             match buffer_mode {
                 InputMode::GotoLine => {
@@ -311,8 +324,8 @@ impl<'a> CsvTable<'a> {
                 content += format!(" {}", s.status_line()).as_str();
             }
 
-            if let Some(elapsed) = state.elapsed {
-                content += format!(" [{}ms]", elapsed).as_str();
+            if let Some(stats_line) = &state.debug_stats.status_line() {
+                content += format!(" {}", stats_line).as_str();
             }
 
             if !state.debug.is_empty() {
@@ -421,7 +434,7 @@ pub struct FinderActiveState {
     find_complete: bool,
     total_found: u64,
     cursor_index: Option<u64>,
-    target: String,
+    target: Regex,
     found_record: Option<find::FoundRecord>,
     selected_offset: Option<u64>,
     is_filter: bool,
@@ -479,6 +492,45 @@ struct BordersState {
     y_first_record: u16,
 }
 
+pub struct DebugStats {
+    rows_view_elapsed: Option<f64>,
+    finder_elapsed: Option<f64>,
+}
+
+impl DebugStats {
+    pub fn new() -> Self {
+        DebugStats {
+            rows_view_elapsed: None,
+            finder_elapsed: None,
+        }
+    }
+
+    pub fn rows_view_elapsed(&mut self, elapsed: Option<u128>) {
+        self.rows_view_elapsed = elapsed.map(|e| e as f64 / 1000.0);
+    }
+
+    pub fn finder_elapsed(&mut self, elapsed: Option<u128>) {
+        self.finder_elapsed = elapsed.map(|e| e as f64 / 1000.0);
+    }
+
+    pub fn status_line(&self) -> Option<String> {
+        let mut line = "[".to_string();
+        if let Some(elapsed) = self.rows_view_elapsed {
+            line += format!("rows:{}ms", elapsed).as_str();
+        }
+        if let Some(elapsed) = self.finder_elapsed {
+            line += format!(" finder:{}ms", elapsed).as_str();
+        }
+        line += "]";
+        if line == "[]" {
+            None
+        }
+        else {
+            Some(line)
+        }
+    }
+}
+
 pub struct CsvTableState {
     // TODO: types appropriate?
     pub rows_offset: u64,
@@ -488,13 +540,14 @@ pub struct CsvTableState {
     filename: Option<String>,
     total_line_number: Option<usize>,
     total_cols: usize,
-    pub elapsed: Option<f64>,
+    pub debug_stats: DebugStats,
     buffer_content: BufferState,
     pub finder_state: FinderState,
     borders_state: Option<BordersState>,
     // TODO: should probably be with BordersState
     col_ending_pos_x: u16,
     pub selected: Option<u64>,
+    pub user_error: Option<String>,
     pub debug: String,
 }
 
@@ -508,12 +561,13 @@ impl CsvTableState {
             filename,
             total_line_number: None,
             total_cols,
-            elapsed: None,
+            debug_stats: DebugStats::new(),
             buffer_content: BufferState::Disabled,
             finder_state: FinderState::FinderInactive,
             borders_state: None,
             col_ending_pos_x: 0,
             selected: None,
+            user_error: None,
             debug: "".into(),
         }
     }
